
#ifndef YOURDETECTORCONSTRUCTION_HH
#define YOURDETECTORCONSTRUCTION_HH

#include "G4VUserDetectorConstruction.hh"

  /**
   * M. Novak (10 May 2022) 
   *
   * Please note, that this is just an intermediate solution that will 
   * be modified later on this week!
   *
   * Please remember, that we want to provide the flexibility for our 
   * users of changing both the target thickness and material. Moreover,
   * we would like to make possible these modifications even between runs, 
   * i.e. even after the detector is already constructed.
   *
   * This version of the implementation provides a `soft` solution that
   * works well ONLY in case of simply setups (like the one we have). 
   * This solution avoids rebuilding the geomerty, i.e. no need to re-invoke
   * the Construct() interface method. It is achieved by 
   * 1. Keeping access to the related:
   *     - shape objects:              fTargetShape, fWorldShape 
   *     - the target logical volume:  fTargetLogical
   *    after their construction in the Construct() method.
   * 2. Re-setting:
   *     - the size parameters of the shape objects when they have been asked 
   *       to be changed (after they are recomuted in the ComputeParameters()
   *       method). The run manager is informed that the geomerty has been 
   *       modified, which triggers the recomputations of some geometry related
   *       optimisations before the next run.
   *       ( G4RunManager::GetRunManager()->GeometryHasBeenModified(); )  
   *     - the material of the target logical volume when it has been aked to 
   *       be changed. The run manager is informed that the physics has been 
   *       modified, which triggers the recomputations of the physics related
   *       data (that depends on the materials) before the next run.   
   *       ( G4RunManager::GetRunManager()->PhysicsHasBeenModified(); )  
   *
   * An other solution that is more general and works fine in all cases, is to
   * simple infom the run manager that the detector needs to be re-constructed.
   * This triggers the complete reinitialisation chain starting with (re-)
   * invoking the Construct() interface method. This can be done either from
   * the code as G4RunManager::GetRunManager()->ReinitializeGeometry(); )
   *
   * The second, more general solution will be used later (so all the first 
   * solution related member variables and codes will be removed)! 
   *
   * Note: it is important to get an insight into the initialisation process so
   *       this is why we decided to show you first this solution. We will face
   *       furder issues later that are related to the initialisation order 
   *       (e.g. the proper gun position in case the target thickness size has
   *        been changed form its default value given in the constructor.)  
   */          


class G4Material;
class G4Box;
class YourDetectorMessenger;

class YourDetectorConstruction : public G4VUserDetectorConstruction {

  public:

   // constructor and destructor
   YourDetectorConstruction();
   ~YourDetectorConstruction();
   
   // this is the interface method to implement the geomerty
   // must return with the ptr to the world physical volume 
   virtual G4VPhysicalVolume* Construct();

  
   //
   // Additionals


   // set/get target material
   void  SetTargetMaterial(const G4String& nistMatName);
   const G4Material* GetTargetMaterial() const { return fTargetMaterial; }

   // set/get target thickness
   void  SetTargetThickness(G4double val);
   G4double GetTargetThickness() const { return fTargetThickness; } 
   

   // computes all shape parameters depending on the target thickness
   void  ComputeParameters();
   

   // get the computed initial x position of the primary particles
   G4double   GetGunXPosition() const { return fGunXPosition; }
   

   // give the target physical for YourSteppingAction
   const G4VPhysicalVolume*  GetTargetPhysicalVolume () const { return fTargetPhysical; } 

  private:
  
   // target material
   G4Material*    fTargetMaterial; 

   // target thickness and YZ extent
   G4double       fTargetThickness;
   G4double       fTargetYZSize;
   // world X size and YZ extent
   G4double       fWorldXSize;
   G4double       fWorldYZSize;

   // initial x-postion of the primary particles
   // (this depends on the target thickness)
   G4double       fGunXPosition;

   // keep the shape ptrs to be able to resize the target
   G4Box*         fTargetShape;
   G4Box*         fWorldShape;
   // keep the target logical to be able to change its material ptr
   G4LogicalVolume*  fTargetLogical;  

   // keep the target physical volume to be able to check if teh step was 
   // done inside the target (used in YourSteppingAction)
   G4VPhysicalVolume* fTargetPhysical;

   YourDetectorMessenger*  fMessenger;

};


#endif  
