

#include "YourDetectorConstruction.hh"

#include "YourDetectorMessenger.hh"

// fo geometry definition 
#include "G4Box.hh"
#include "G4LogicalVolume.hh"
#include "G4PVPlacement.hh"

// for matrial definition
#include "G4Material.hh"
#include "G4NistManager.hh"

// just to be able to inform that physics/geomerty 
// has been modified (if needed at all!)
#include "G4RunManager.hh"

// for units
#include "G4SystemOfUnits.hh"


YourDetectorConstruction::YourDetectorConstruction() 
  : G4VUserDetectorConstruction(),
    fTargetMaterial(nullptr),
    fTargetShape(nullptr),
    fWorldShape(nullptr),
    fTargetLogical(nullptr),
    fTargetPhysical(nullptr){

    const G4String matName = "G4_Si";
    SetTargetMaterial(matName);

    fTargetThickness = 1.0*CLHEP::cm; 
    ComputeParameters();

    fMessenger = new YourDetectorMessenger(this);
}


YourDetectorConstruction::~YourDetectorConstruction() { 
   delete fMessenger;

}


void YourDetectorConstruction::SetTargetMaterial(const G4String& name) {
    fTargetMaterial = G4NistManager::Instance()->FindOrBuildMaterial(name);
    if (fTargetMaterial == nullptr) {
      G4cerr << " *** ERROR YourDetectorConstruction: unknown material = " 
             << name 
             << G4endl;
      exit(-1);  
    } 
    G4RunManager::GetRunManager()->ReinitializeGeometry();
    //
    // NOTE: this is to change the target material without
    //       rebuilding the detector. Physics needs to be 
    //       reinitialised!
/*
    if (fTargetLogical != nullptr) {
      // reset the target material (of the target logical volume
      fTargetLogical->SetMaterial(fTargetMaterial);
      // inform the run manager that geomrty has been modified
      G4RunManager::GetRunManager()->PhysicsHasBeenModified();
    }
*/
}


void YourDetectorConstruction::SetTargetThickness(G4double val) {
   fTargetThickness = val;
   ComputeParameters();
}


void YourDetectorConstruction::ComputeParameters() {
    fTargetYZSize = 5.0*fTargetThickness;
    fWorldXSize   = 1.5*fTargetThickness;
    fWorldYZSize  = 1.5*fTargetYZSize;
    // compute gun x position
    fGunXPosition = -0.25*(fWorldXSize + fTargetThickness); 
    // 
     G4RunManager::GetRunManager()->ReinitializeGeometry();
    // NOTE: this is to change the target thickness without
    //       rebuilding the detector. Physics not but the
    //       geometry can be re-initialised, which means 
    //       only the re-computation of some geomerty 
    //       optimisations and NOT the compete re-building 
    //       of the detector.
/*
    if (fWorldShape != nullptr) {
      // reset the world shape (box) size parameters
      fWorldShape->SetXHalfLength(0.5*fWorldXSize);
      fWorldShape->SetYHalfLength(0.5*fWorldYZSize);
      fWorldShape->SetZHalfLength(0.5*fWorldYZSize);
      // reset the target shape (box) size parameters
      fTargetShape->SetXHalfLength(0.5*fTargetThickness);
      fTargetShape->SetYHalfLength(0.5*fTargetYZSize);
      fTargetShape->SetZHalfLength(0.5*fTargetYZSize);
      // inform the run manager that geomrty has been modified
      G4RunManager::GetRunManager()->GeometryHasBeenModified();
    }   
*/
}


G4VPhysicalVolume* YourDetectorConstruction::Construct() {
   // create material for the world
   G4Material* wMat = G4NistManager::Instance()->FindOrBuildMaterial("G4_Galactic"); 
   //
   // world
   fWorldShape                 = new G4Box("solid-world", 0.5*fWorldXSize, 0.5*fWorldYZSize, 0.5*fWorldYZSize);
   G4LogicalVolume*   wLogical = new G4LogicalVolume(fWorldShape, wMat, "logic-world");
   G4VPhysicalVolume* wPhys    = new G4PVPlacement(nullptr, G4ThreeVector(0,0,0), wLogical, "World", nullptr, false, 0);
   //
   // target
   fTargetShape    = new G4Box("solid-target", 0.5*fTargetThickness, 0.5*fTargetYZSize, 0.5*fTargetYZSize);
   fTargetLogical  = new G4LogicalVolume(fTargetShape, fTargetMaterial, "logic-target");
   fTargetPhysical = new G4PVPlacement(nullptr, G4ThreeVector(0,0,0), fTargetLogical, "Target", wLogical, false, 0);
     
  // return the pointer of the world physical volume   
  return wPhys;
}






