
// **************************************************************************** 
//
// Author: Mihaly Novak 
// Date  : 13 May 2022 
// -------------------
// 
// Description: see the `README.txt` 
// ------------
//
// **************************************************************************** 


// for some Geant4 types and io
#include "globals.hh"


// to obtain a run manager, the top level manager
// - where we need to register our detector, physics
//   and action objects after their construction
// - that controls the complete flow of initialisation
//   and simulation 
#include "G4RunManagerFactory.hh"


// for our detector implementation (mandatory)
#include "YourDetectorConstruction.hh"

// for our action implementation (mandatory) 
// this includes our primary generator action
#include "YourActionInitialization.hh"

// for the physics list (mandatory)
// (we use a reference physics list to keep is simple)
#include "G4PhysListFactory.hh"

// to able to use units e.g. CLHEP::cm
#include "G4SystemOfUnits.hh"


// for command execution: we will use this to execute the input macro file; if any
#include "G4UImanager.hh"
// for interactive UI sessions (Qt-GUI, tcsh)
#include "G4UIExecutive.hh"


// for visualization 
#include "G4VisManager.hh" 
#include "G4VisExecutive.hh"



int main(int argc, char** argv) {

  G4String macroFile = "";
  G4UIExecutive* ui  = nullptr;
  if ( argc == 1 ) {
     // no macro file passed by the user at execution (as `./yourMain`)
//     ui = new G4UIExecutive(argc, argv, "tcsh"); 
     ui = new G4UIExecutive(argc, argv); 
  } else {
     // macro file was passed by the user at execution (as `./yourMain g4macro`)
     macroFile = argv [1];
  } 

  // ==============================================================================
  // Create the run manager then construct all mandatory user initializations and 
  // register them in the run manager  
  // ==============================================================================

  // Construct the run manager
  auto* runMgr = G4RunManagerFactory::CreateRunManager();

  // Construct our detector description object and register in the run manager
  YourDetectorConstruction* det = new YourDetectorConstruction();
  runMgr->SetUserInitialization( det );  

  // Construct a (reference: "FTFP_BERT") physics list and register in the run manager 
  const G4String plName = "FTFP_BERT";  // "FTFP_BERT_EMZ"
  G4PhysListFactory plFactory;
  G4VModularPhysicsList* pl= plFactory.GetReferencePhysList(plName); 
  runMgr->SetUserInitialization( pl );
   
  // Construct our action initialization and register in the run manager 
  // ------------------------------------------------------------------------------
  // NOTE: it's action INITIALIZATION since it encapsulates the construction and 
  //       registration of ALL user ACTIONS (at least the primary generator action)
  YourActionInitialization* action = new YourActionInitialization(det);
  runMgr->SetUserInitialization( action );


  // ==============================================================================
  // The bare minimum setup is ready at this point, the simulation can be 
  // initialized and executed now
  // ==============================================================================

  
   G4VisManager* visMgr = new G4VisExecutive();
   visMgr->Initialize();


  if ( ui != nullptr ) {
    // interactive: start the interactive session (GUI or shell)
    //              where commands can be executed  
    ui -> SessionStart();
  } else {
    // batch mode: execute the macro file that was the input argument
    G4String cmd = "/control/execute ";
    G4UImanager::GetUIpointer()->ApplyCommand(cmd + macroFile);
  }


  // delete vis manager
  delete visMgr;

  // Delete the run manager at the end of the run
  delete runMgr;
  
  return 0;
}

